'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.layoutGraph = layoutGraph;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _PipelineGraphModel = require('./PipelineGraphModel');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Main process for laying out the graph. Creates and positions markers for each component, but creates no components.
 *
 *  1. Creates nodes for each stage in the pipeline
 *  2. Position the nodes in columns for each top stage, and in rows within each column based on execution order
 *  3. Create all the connections between nodes that need to be rendered
 *  4. Create a bigLabel per column, and a smallLabel for any child nodes
 *  5. Measure the extents of the graph
 */
function layoutGraph(newStages, layout) {
    var stageNodeColumns = createNodeColumns(newStages);
    var nodeSpacingH = layout.nodeSpacingH,
        ypStart = layout.ypStart;


    var startNode = {
        x: 0,
        y: 0,
        name: 'Start',
        id: -1,
        isPlaceholder: true,
        key: 'start-node',
        type: 'start'
    };

    var endNode = {
        x: 0,
        y: 0,
        name: 'End',
        id: -2,
        isPlaceholder: true,
        key: 'end-node',
        type: 'end'
    };

    var allNodeColumns = [{ rows: [[startNode]], x: 0 }].concat(stageNodeColumns, [{ rows: [[endNode]], x: 0 }]);

    positionNodes(allNodeColumns, layout);

    var bigLabels = createBigLabels(allNodeColumns);
    var smallLabels = createSmallLabels(allNodeColumns);
    var connections = createConnections(allNodeColumns);

    // Calculate the size of the graph
    var measuredWidth = 0;
    var measuredHeight = 200;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = allNodeColumns[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var column = _step.value;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = column.rows[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var row = _step2.value;
                    var _iteratorNormalCompletion3 = true;
                    var _didIteratorError3 = false;
                    var _iteratorError3 = undefined;

                    try {
                        for (var _iterator3 = row[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                            var node = _step3.value;

                            measuredWidth = Math.max(measuredWidth, node.x + nodeSpacingH / 2);
                            measuredHeight = Math.max(measuredHeight, node.y + ypStart);
                        }
                    } catch (err) {
                        _didIteratorError3 = true;
                        _iteratorError3 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                                _iterator3.return();
                            }
                        } finally {
                            if (_didIteratorError3) {
                                throw _iteratorError3;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }

        // TODO: Type
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return {
        nodeColumns: allNodeColumns,
        connections: connections,
        bigLabels: bigLabels,
        smallLabels: smallLabels,
        measuredWidth: measuredWidth,
        measuredHeight: measuredHeight
    };
}

/**
 * Generate an array of columns, based on the top-level stages
 */
function createNodeColumns() {
    var topLevelStages = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    var nodeColumns = [];

    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
        for (var _iterator4 = topLevelStages[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var topStage = _step4.value;

            // If stage has children, we don't draw a node for it, just its children
            var stagesForColumn = topStage.children && topStage.children.length ? topStage.children : [topStage];

            var column = {
                topStage: topStage,
                rows: [],
                x: 0 // Layout is done later
            };

            var _iteratorNormalCompletion5 = true;
            var _didIteratorError5 = false;
            var _iteratorError5 = undefined;

            try {
                for (var _iterator5 = stagesForColumn[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                    var firstStageForRow = _step5.value;

                    if (firstStageForRow) {
                        // Skip incomplete data generated by server response limits
                        var rowNodes = [];
                        var nodeStage = firstStageForRow;

                        while (nodeStage) {
                            rowNodes.push({
                                x: 0, // Layout is done later
                                y: 0,
                                name: nodeStage.name,
                                id: nodeStage.id,
                                stage: nodeStage,
                                isPlaceholder: false,
                                key: 'n_' + nodeStage.id
                            });
                            nodeStage = nodeStage.nextSibling;
                        }
                        column.rows.push(rowNodes);
                    }
                }
            } catch (err) {
                _didIteratorError5 = true;
                _iteratorError5 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion5 && _iterator5.return) {
                        _iterator5.return();
                    }
                } finally {
                    if (_didIteratorError5) {
                        throw _iteratorError5;
                    }
                }
            }

            if (_PipelineGraphModel.MATRIOSKA_PATHS) {
                (function () {
                    // Sort by row length for visual appeal when connectors don't all line up vertically
                    var originalOrder = column.rows.concat();
                    column.rows.sort(function (left, right) {
                        if (left.length < right.length) {
                            return -1;
                        }
                        if (left.length > right.length) {
                            return 1;
                        }
                        return originalOrder.indexOf(left) < originalOrder.indexOf(right) ? -1 : 1; // For stability
                    });
                })();
            }

            nodeColumns.push(column);
        }
    } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
            }
        } finally {
            if (_didIteratorError4) {
                throw _iteratorError4;
            }
        }
    }

    return nodeColumns;
}

/**
 * Walks the columns of nodes giving them x and y positions. Mutates the node objects in place for now.
 */
function positionNodes(nodeColumns, _ref) {
    var nodeSpacingH = _ref.nodeSpacingH,
        parallelSpacingH = _ref.parallelSpacingH,
        nodeSpacingV = _ref.nodeSpacingV,
        ypStart = _ref.ypStart;

    var xp = nodeSpacingH / 2;
    var previousTopNode = null;

    var _iteratorNormalCompletion6 = true;
    var _didIteratorError6 = false;
    var _iteratorError6 = undefined;

    try {
        for (var _iterator6 = nodeColumns[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
            var column = _step6.value;

            var topNode = column.rows[0][0];

            var yp = ypStart; // Reset Y to top for each column

            if (previousTopNode) {
                // Advance X position
                if (previousTopNode.isPlaceholder || topNode.isPlaceholder) {
                    // Don't space placeholder nodes (start/end) as wide as normal.
                    xp += Math.floor(nodeSpacingH * 0.7);
                } else {
                    xp += nodeSpacingH;
                }
            }

            var widestRow = 0;
            var _iteratorNormalCompletion7 = true;
            var _didIteratorError7 = false;
            var _iteratorError7 = undefined;

            try {
                for (var _iterator7 = column.rows[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                    var row = _step7.value;

                    widestRow = Math.max(widestRow, row.length);
                }
            } catch (err) {
                _didIteratorError7 = true;
                _iteratorError7 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion7 && _iterator7.return) {
                        _iterator7.return();
                    }
                } finally {
                    if (_didIteratorError7) {
                        throw _iteratorError7;
                    }
                }
            }

            var xpStart = xp; // Left-most position in this column
            var maxX = xp;

            var _iteratorNormalCompletion8 = true;
            var _didIteratorError8 = false;
            var _iteratorError8 = undefined;

            try {
                for (var _iterator8 = column.rows[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                    var _row = _step8.value;

                    // Offset the beginning of narrower rows towards column center
                    xp += Math.round((widestRow - _row.length) * parallelSpacingH * 0.5);

                    var _iteratorNormalCompletion9 = true;
                    var _didIteratorError9 = false;
                    var _iteratorError9 = undefined;

                    try {
                        for (var _iterator9 = _row[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
                            var node = _step9.value;

                            maxX = Math.max(maxX, xp);
                            node.x = xp;
                            node.y = yp;
                            xp += parallelSpacingH; // Space out nodes in each row
                        }
                    } catch (err) {
                        _didIteratorError9 = true;
                        _iteratorError9 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion9 && _iterator9.return) {
                                _iterator9.return();
                            }
                        } finally {
                            if (_didIteratorError9) {
                                throw _iteratorError9;
                            }
                        }
                    }

                    xp = xpStart; // CR
                    yp += nodeSpacingV; // LF
                }
            } catch (err) {
                _didIteratorError8 = true;
                _iteratorError8 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion8 && _iterator8.return) {
                        _iterator8.return();
                    }
                } finally {
                    if (_didIteratorError8) {
                        throw _iteratorError8;
                    }
                }
            }

            column.x = Math.round((xpStart + maxX) / 2); // Center of column
            xp = maxX; // Make sure we're at the end of the widest row for this column before next loop

            previousTopNode = topNode;
        }
    } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion6 && _iterator6.return) {
                _iterator6.return();
            }
        } finally {
            if (_didIteratorError6) {
                throw _iteratorError6;
            }
        }
    }
}

/**
 * Generate label descriptions for big labels at the top of each column
 */
function createBigLabels(columns) {
    var labels = [];

    var _iteratorNormalCompletion10 = true;
    var _didIteratorError10 = false;
    var _iteratorError10 = undefined;

    try {
        for (var _iterator10 = columns[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
            var column = _step10.value;

            var node = column.rows[0][0];
            var stage = column.topStage;
            var text = stage ? stage.name : node.name;
            var key = 'l_b_' + node.key;

            labels.push({
                x: column.x, // bigLabel is located above center of column
                y: node.y,
                node: node,
                stage: stage,
                text: text,
                key: key
            });
        }
    } catch (err) {
        _didIteratorError10 = true;
        _iteratorError10 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion10 && _iterator10.return) {
                _iterator10.return();
            }
        } finally {
            if (_didIteratorError10) {
                throw _iteratorError10;
            }
        }
    }

    return labels;
}

/**
 * Generate label descriptions for small labels under the nodes
 */
function createSmallLabels(columns) {
    var labels = [];

    var _iteratorNormalCompletion11 = true;
    var _didIteratorError11 = false;
    var _iteratorError11 = undefined;

    try {
        for (var _iterator11 = columns[Symbol.iterator](), _step11; !(_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done); _iteratorNormalCompletion11 = true) {
            var column = _step11.value;
            var _iteratorNormalCompletion12 = true;
            var _didIteratorError12 = false;
            var _iteratorError12 = undefined;

            try {
                for (var _iterator12 = column.rows[Symbol.iterator](), _step12; !(_iteratorNormalCompletion12 = (_step12 = _iterator12.next()).done); _iteratorNormalCompletion12 = true) {
                    var row = _step12.value;
                    var _iteratorNormalCompletion13 = true;
                    var _didIteratorError13 = false;
                    var _iteratorError13 = undefined;

                    try {
                        for (var _iterator13 = row[Symbol.iterator](), _step13; !(_iteratorNormalCompletion13 = (_step13 = _iterator13.next()).done); _iteratorNormalCompletion13 = true) {
                            var node = _step13.value;

                            // We add small labels to parallel nodes only so skip others
                            if (!node.stage || node.stage.type !== 'PARALLEL' && node.stage.isSequential !== true) {
                                continue;
                            }
                            var label = {
                                x: node.x,
                                y: node.y,
                                text: node.name,
                                key: 'l_s_' + node.key,
                                node: node
                            };

                            if (node.isPlaceholder === false) {
                                label.stage = node.stage;
                            }

                            labels.push(label);
                        }
                    } catch (err) {
                        _didIteratorError13 = true;
                        _iteratorError13 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion13 && _iterator13.return) {
                                _iterator13.return();
                            }
                        } finally {
                            if (_didIteratorError13) {
                                throw _iteratorError13;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError12 = true;
                _iteratorError12 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion12 && _iterator12.return) {
                        _iterator12.return();
                    }
                } finally {
                    if (_didIteratorError12) {
                        throw _iteratorError12;
                    }
                }
            }
        }
    } catch (err) {
        _didIteratorError11 = true;
        _iteratorError11 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion11 && _iterator11.return) {
                _iterator11.return();
            }
        } finally {
            if (_didIteratorError11) {
                throw _iteratorError11;
            }
        }
    }

    return labels;
}

/**
 * Generate connection information from column to column
 */
function createConnections(columns) {
    var connections = [];

    var sourceNodes = [];
    var skippedNodes = [];

    var _iteratorNormalCompletion14 = true;
    var _didIteratorError14 = false;
    var _iteratorError14 = undefined;

    try {
        for (var _iterator14 = columns[Symbol.iterator](), _step14; !(_iteratorNormalCompletion14 = (_step14 = _iterator14.next()).done); _iteratorNormalCompletion14 = true) {
            var column = _step14.value;

            if (column.topStage && column.topStage.state === 'skipped') {
                skippedNodes.push(column.rows[0][0]);
                continue;
            }

            // Connections to each row in this column
            if (sourceNodes.length) {
                connections.push({
                    sourceNodes: sourceNodes,
                    destinationNodes: column.rows.map(function (row) {
                        return row[0];
                    }), // First node of each row
                    skippedNodes: skippedNodes
                });
            }

            // Connections between nodes within each row
            var _iteratorNormalCompletion15 = true;
            var _didIteratorError15 = false;
            var _iteratorError15 = undefined;

            try {
                for (var _iterator15 = column.rows[Symbol.iterator](), _step15; !(_iteratorNormalCompletion15 = (_step15 = _iterator15.next()).done); _iteratorNormalCompletion15 = true) {
                    var row = _step15.value;

                    for (var i = 0; i < row.length - 1; i++) {
                        connections.push({
                            sourceNodes: [row[i]],
                            destinationNodes: [row[i + 1]],
                            skippedNodes: []
                        });
                    }
                }
            } catch (err) {
                _didIteratorError15 = true;
                _iteratorError15 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion15 && _iterator15.return) {
                        _iterator15.return();
                    }
                } finally {
                    if (_didIteratorError15) {
                        throw _iteratorError15;
                    }
                }
            }

            sourceNodes = column.rows.map(function (row) {
                return row[row.length - 1];
            }); // Last node of each row
            skippedNodes = [];
        }
    } catch (err) {
        _didIteratorError14 = true;
        _iteratorError14 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion14 && _iterator14.return) {
                _iterator14.return();
            }
        } finally {
            if (_didIteratorError14) {
                throw _iteratorError14;
            }
        }
    }

    return connections;
}
//# sourceMappingURL=PipelineGraphLayout.js.map
